/* TinyBasic (Lex/Flex lexer definition file.)
 * Version 0.1 (13/09-08)
 * Copyright 2008 Kjetil Erga (kobolt.anarion -AT- gmail -DOT- com)
 *
 *
 * This file is part of TinyBasic.
 *
 * TinyBasic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * TinyBasic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TinyBasic. If not, see <http://www.gnu.org/licenses/>.
 */

%{
#include "y.tab.h"
#include <stdlib.h>
#include <string.h>

#define YY_DECL int yylex(YYSTYPE *lvalp)

void yyerror(char *s);
%}

%option nounput
%option case-insensitive

%%

#.*        ; /* Ignore unix-style comments. */

"REM".*$   { lvalp->string = malloc(strlen(yytext) * sizeof(char));
             if (lvalp->string == NULL)
               yyerror("out of memory");
             memset(lvalp->string, '\0', strlen(yytext));
             /* Remove "REM" text also. */
             strncpy(lvalp->string, yytext + 3, strlen(yytext) - 3);
             return REM; }

"PRINT"    { return PRINT; }
"IF"       { return IF; }
"THEN"     { return THEN; }
"GOTO"     { return GOTO; }
"INPUT"    { return INPUT; }
"LET"      { return LET; }
"GOSUB"    { return GOSUB; }
"RETURN"   { return RETURN; }
"CLEAR"    { return CLEAR; }
"LIST"     { return LIST; }
"RUN"      { return RUN; }
"END"      { return END; }
"RND"      { return RND; }
"USR"      { return USR; }
"LOAD"     { return LOAD; }
"SAVE"     { return SAVE; }
"TREELIST" { return TREELIST; }
"SYSTEM"   { return SYSTEM; }

"+"        { return '+'; }
"-"        { return '-'; }
"*"        { return '*'; }
"/"        { return '/'; }

"("        { return '('; }
")"        { return ')'; }

"="        { return EQ; }
"<>"       { return NE; }
"<"        { return LT; }
"<="       { return LE; }
">"        { return GT; }
">="       { return GE; }

[,;][ \t]*$ ; /* Ignore trailing comma's. (Also with tabs and space behind.) */

","        { return COMMA; }
";"        { return COMMA; }

[\n\r]     { return NEWLINE; }

[ \t]+     ; /* Ignore tabs and spaces. */

[A-Z]{1}   { lvalp->var = yytext[0]; return VAR; }
[0-9]+	   { lvalp->number = atoi(yytext); return NUMBER; }
\"[^\"]*\" { lvalp->string = malloc(strlen(yytext) * sizeof(char));
             if (lvalp->string == NULL)
               yyerror("out of memory");
             memset(lvalp->string, '\0', strlen(yytext));
             /* Remove surrounding double quotes also. */
             strncpy(lvalp->string, yytext + 1, strlen(yytext) - 2);
             return STRING; }

.          { yyerror("illegal token"); }

%%

int yywrap(void) { return 1; }

void lex_file_read_start(FILE *fh)
{
  YY_BUFFER_STATE buffer;
  buffer = yy_create_buffer(fh, YY_BUF_SIZE);
  yypush_buffer_state(buffer);
}

void lex_file_read_stop(void)
{
  yypop_buffer_state();
}

